@@ -277,6 +277,7 @@ enum rgmii_clock_delay {
 /* Microsemi PHY ID's
  *   Code assumes lowest nibble is 0
  */
+#define PHY_ID_VSC8501			  0x00070530
 #define PHY_ID_VSC8502			  0x00070630
 #define PHY_ID_VSC8504			  0x000704c0
 #define PHY_ID_VSC8514			  0x00070670
@@ -291,6 +292,7 @@ enum rgmii_clock_delay {
 #define PHY_ID_VSC8575			  0x000707d0
 #define PHY_ID_VSC8582			  0x000707b0
 #define PHY_ID_VSC8584			  0x000707c0
+#define PHY_VENDOR_MSCC			0x00070400
 
 #define MSCC_VDDMAC_1500		  1500
 #define MSCC_VDDMAC_1800		  1800
@@ -360,13 +362,6 @@ struct vsc85xx_hw_stat {
 	u16 mask;
 };
 
-struct vsc8531_skb_cb {
-	u32 ns;
-};
-
-#define VSC8531_SKB_CB(skb) \
-	((struct vsc8531_skb_cb *)((skb)->cb))
-
 struct vsc8531_private {
 	int rate_magic;
 	u16 supp_led_modes;
@@ -415,11 +410,6 @@ struct vsc8531_private {
 	 */
 	struct mutex ts_lock;
 	struct mutex phc_lock;
-
-	/* list of skbs that were received and need timestamp information but it
-	 * didn't received it yet
-	 */
-	struct sk_buff_head rx_skbs_list;
 };
 
 /* Shared structure between the PHYs of the same package.
@@ -474,7 +464,6 @@ static inline void vsc8584_config_macsec_intr(struct phy_device *phydev)
 void vsc85xx_link_change_notify(struct phy_device *phydev);
 void vsc8584_config_ts_intr(struct phy_device *phydev);
 int vsc8584_ptp_init(struct phy_device *phydev);
-void vsc8584_ptp_deinit(struct phy_device *phydev);
 int vsc8584_ptp_probe_once(struct phy_device *phydev);
 int vsc8584_ptp_probe(struct phy_device *phydev);
 irqreturn_t vsc8584_handle_ts_interrupt(struct phy_device *phydev);
@@ -489,9 +478,6 @@ static inline int vsc8584_ptp_init(struct phy_device *phydev)
 {
 	return 0;
 }
-static inline void vsc8584_ptp_deinit(struct phy_device *phydev)
-{
-}
 static inline int vsc8584_ptp_probe_once(struct phy_device *phydev)
 {
 	return 0;
